@@ -1,6 +1,6 @@
 /*
  * activity.c: Define system activities available for sar/sadc.
- * (C) 1999-2008 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2010 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -20,10 +20,18 @@
  */
 
 #include "sa.h"
-#include "rd_stats.h"
-#include "pr_stats.h"
-#include "prf_stats.h"
 
+#ifdef SOURCE_SADC
+#include "rd_stats.h"
+#endif
+
+#ifdef SOURCE_SAR
+#include "pr_stats.h"
+#endif
+
+#ifdef SOURCE_SADF
+#include "prf_stats.h"
+#endif
 
 /*
  ***************************************************************************
@@ -33,31 +41,23 @@
  */
 
 /*
- * Array of item numbers (CPUs, network interfaces, etc.)
- * A negative value (-1) is the default value and indicates that this number
- * has still not been calculated by the f_count() function from the corresponding
- * activity.
- * A value of 0 means that this number has been calculated, but no items have
- * been found.
+ * Bitmaps needed by activities.
+ * Remember to allocate them before use!
  */
-__nr_t item_nr[] = {
-	-1,	/*  [0] Filled by wrap_get_cpu_nr() for A_CPU				*/
-	 1,	/*  [1] Constant for A_PCSW						*/
-	-1,	/*  [2] Filled by wrap_get_irq_nr() for A_IRQ				*/
-	-1,	/*  [3] Filled by wrap_get_serial_nr() for A_SERIAL			*/
-	-1,	/*  [4] Filled by wrap_get_disk_nr() for A_DISK				*/
-	-1,	/*  [5] Filled by wrap_get_iface_nr() for A_NET_DEV and A_NET_EDEV	*/
-	 1,	/*  [6] Constant for A_SWAP						*/
-	 1,	/*  [7] Constant for A_PAGE						*/
-	 1,	/*  [8] Constant for A_IO						*/
-	 1,	/*  [9] Constant for A_MEMORY						*/
-	 1,	/* [10] Constant for A_KTABLES						*/
-	 1,	/* [11] Constant for A_QUEUE						*/
-	 1,	/* [12] Constant for A_NET_NFS and A_NET_NFSD				*/
-	 1,	/* [13] Constant for A_NET_SOCK						*/
-	 1	/* [14] Constant for SNMP statistics (A_NET_IP, etc.)			*/
+
+/* CPU bitmap */
+struct act_bitmap cpu_bitmap = {
+	.b_array	= NULL,
+	.b_size		= NR_CPUS
 };
 
+/* Interrupts bitmap */
+struct act_bitmap irq_bitmap = {
+	.b_array	= NULL,
+	.b_size		= NR_IRQS
+};
+
+
 /*
  * CPU statistics.
  * This is the only activity which *must* be collected by sadc
@@ -81,13 +81,12 @@ struct activity cpu_act = {
 		          "CPU;%usr;%nice;%sys;%iowait;%steal;%irq;%soft;%guest;%idle",
 	.name		= "A_CPU",
 #endif
-	.nr		= &item_nr[0],
+	.nr		= -1,
 	.fsize		= STATS_CPU_SIZE,
 	.msize		= STATS_CPU_SIZE,
 	.opt_flags	= AO_F_CPU_DEF,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= NR_CPUS
+	.bitmap		= &cpu_bitmap
 };
 
 /* Process (task) creation and context switch activity */
@@ -108,13 +107,12 @@ struct activity pcsw_act = {
 	.hdr_line	= "proc/s;cswch/s",
 	.name		= "A_PCSW",
 #endif
-	.nr		= &item_nr[1],
+	.nr		= 1,
 	.fsize		= STATS_PCSW_SIZE,
 	.msize		= STATS_PCSW_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* Interrupts statistics */
@@ -135,13 +133,12 @@ struct activity irq_act = {
 	.hdr_line	= "INTR;intr/s",
 	.name		= "A_IRQ",
 #endif
-	.nr		= &item_nr[2],
+	.nr		= -1,
 	.fsize		= STATS_IRQ_SIZE,
 	.msize		= STATS_IRQ_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= NR_IRQS
+	.bitmap		= &irq_bitmap
 };
 
 /* Swapping activity */
@@ -162,13 +159,12 @@ struct activity swap_act = {
 	.hdr_line	= "pswpin/s;pswpout/s",
 	.name		= "A_SWAP",
 #endif
-	.nr		= &item_nr[6],
+	.nr		= 1,
 	.fsize		= STATS_SWAP_SIZE,
 	.msize		= STATS_SWAP_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* Paging activity */
@@ -190,13 +186,12 @@ struct activity paging_act = {
 		          "pgfree/s;pgscank/s;pgscand/s;pgsteal/s;%vmeff",
 	.name		= "A_PAGE",
 #endif
-	.nr		= &item_nr[7],
+	.nr		= 1,
 	.fsize		= STATS_PAGING_SIZE,
 	.msize		= STATS_PAGING_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* I/O and transfer rate activity */
@@ -217,13 +212,12 @@ struct activity io_act = {
 	.hdr_line	= "tps;rtps;wtps;bread/s;bwrtn/s",
 	.name		= "A_IO",
 #endif
-	.nr		= &item_nr[8],
+	.nr		= 1,
 	.fsize		= STATS_IO_SIZE,
 	.msize		= STATS_IO_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* Memory and swap space utilization activity */
@@ -246,13 +240,12 @@ struct activity memory_act = {
 		          "kbswpfree;kbswpused;%swpused;kbswpcad;%swpcad",
 	.name		= "A_MEMORY",
 #endif
-	.nr		= &item_nr[9],
+	.nr		= 1,
 	.fsize		= STATS_MEMORY_SIZE,
 	.msize		= STATS_MEMORY_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* Kernel tables activity */
@@ -273,13 +266,12 @@ struct activity ktables_act = {
 	.hdr_line	= "dentunusd;file-nr;inode-nr;pty-nr",
 	.name		= "A_KTABLES",
 #endif
-	.nr		= &item_nr[10],
+	.nr		= 1,
 	.fsize		= STATS_KTABLES_SIZE,
 	.msize		= STATS_KTABLES_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* Queue and load activity */
@@ -300,13 +292,12 @@ struct activity queue_act = {
 	.hdr_line	= "runq-sz;plist-sz;ldavg-1;ldavg-5;ldavg-15",
 	.name		= "A_QUEUE",
 #endif
-	.nr		= &item_nr[11],
+	.nr		= 1,
 	.fsize		= STATS_QUEUE_SIZE,
 	.msize		= STATS_QUEUE_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* Serial lines activity */
@@ -327,13 +318,12 @@ struct activity serial_act = {
 	.hdr_line	= "TTY;rcvin/s;txmtin/s;framerr/s;prtyerr/s;brk/s;ovrun/s",
 	.name		= "A_SERIAL",
 #endif
-	.nr		= &item_nr[3],
+	.nr		= -1,
 	.fsize		= STATS_SERIAL_SIZE,
 	.msize		= STATS_SERIAL_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* Block devices activity */
@@ -354,13 +344,12 @@ struct activity disk_act = {
 	.hdr_line	= "DEV;tps;rd_sec/s;wr_sec/s;avgrq-sz;avgqu-sz;await;svctm;%util",
 	.name		= "A_DISK",
 #endif
-	.nr		= &item_nr[4],
+	.nr		= -1,
 	.fsize		= STATS_DISK_SIZE,
 	.msize		= STATS_DISK_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* Network interfaces activity */
@@ -381,13 +370,12 @@ struct activity net_dev_act = {
 	.hdr_line	= "IFACE;rxpck/s;txpck/s;rxkB/s;txkB/s;rxcmp/s;txcmp/s;rxmcst/s",
 	.name		= "A_NET_DEV",
 #endif
-	.nr		= &item_nr[5],
+	.nr		= -1,
 	.fsize		= STATS_NET_DEV_SIZE,
 	.msize		= STATS_NET_DEV_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* Network interfaces activity */
@@ -409,13 +397,12 @@ struct activity net_edev_act = {
 		          "txcarr/s;rxfram/s;rxfifo/s;txfifo/s",
 	.name		= "A_NET_EDEV",
 #endif
-	.nr		= &item_nr[5],
+	.nr		= -1,
 	.fsize		= STATS_NET_EDEV_SIZE,
 	.msize		= STATS_NET_EDEV_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* NFS client activity */
@@ -436,13 +423,12 @@ struct activity net_nfs_act = {
 	.hdr_line	= "call/s;retrans/s;read/s;write/s;access/s;getatt/s",
 	.name		= "A_NET_NFS",
 #endif
-	.nr		= &item_nr[12],
+	.nr		= 1,
 	.fsize		= STATS_NET_NFS_SIZE,
 	.msize		= STATS_NET_NFS_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* NFS server activity */
@@ -464,13 +450,12 @@ struct activity net_nfsd_act = {
 		          "sread/s;swrite/s;saccess/s;sgetatt/s",
 	.name		= "A_NET_NFSD",
 #endif
-	.nr		= &item_nr[12],
+	.nr		= 1,
 	.fsize		= STATS_NET_NFSD_SIZE,
 	.msize		= STATS_NET_NFSD_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* Network sockets activity */
@@ -491,13 +476,12 @@ struct activity net_sock_act = {
 	.hdr_line	= "totsck;tcpsck;udpsck;rawsck;ip-frag;tcp-tw",
 	.name		= "A_NET_SOCK",
 #endif
-	.nr		= &item_nr[13],
+	.nr		= 1,
 	.fsize		= STATS_NET_SOCK_SIZE,
 	.msize		= STATS_NET_SOCK_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* IP network traffic activity */
@@ -518,13 +502,12 @@ struct activity net_ip_act = {
 	.hdr_line	= "irec/s;fwddgm/s;idel/s;orq/s;asmrq/s;asmok/s;fragok/s;fragcrt/s",
 	.name		= "A_NET_IP",
 #endif
-	.nr		= &item_nr[14],
+	.nr		= 1,
 	.fsize		= STATS_NET_IP_SIZE,
 	.msize		= STATS_NET_IP_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* IP network traffic (errors) activity */
@@ -545,13 +528,12 @@ struct activity net_eip_act = {
 	.hdr_line	= "ihdrerr/s;iadrerr/s;iukwnpr/s;idisc/s;odisc/s;onort/s;asmf/s;fragf/s",
 	.name		= "A_NET_EIP",
 #endif
-	.nr		= &item_nr[14],
+	.nr		= 1,
 	.fsize		= STATS_NET_EIP_SIZE,
 	.msize		= STATS_NET_EIP_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* ICMP network traffic activity */
@@ -573,13 +555,12 @@ struct activity net_icmp_act = {
 		          "otmr/s;iadrmk/s;iadrmkr/s;oadrmk/s;oadrmkr/s",
 	.name		= "A_NET_ICMP",
 #endif
-	.nr		= &item_nr[14],
+	.nr		= 1,
 	.fsize		= STATS_NET_ICMP_SIZE,
 	.msize		= STATS_NET_ICMP_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* ICMP network traffic (errors) activity */
@@ -601,13 +582,12 @@ struct activity net_eicmp_act = {
 		          "iparmpb/s;oparmpb/s;isrcq/s;osrcq/s;iredir/s;oredir/s",
 	.name		= "A_NET_EICMP",
 #endif
-	.nr		= &item_nr[14],
+	.nr		= 1,
 	.fsize		= STATS_NET_EICMP_SIZE,
 	.msize		= STATS_NET_EICMP_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* TCP network traffic activity */
@@ -628,13 +608,12 @@ struct activity net_tcp_act = {
 	.hdr_line	= "active/s;passive/s;iseg/s;oseg/s",
 	.name		= "A_NET_TCP",
 #endif
-	.nr		= &item_nr[14],
+	.nr		= 1,
 	.fsize		= STATS_NET_TCP_SIZE,
 	.msize		= STATS_NET_TCP_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* TCP network traffic (errors) activity */
@@ -655,19 +634,18 @@ struct activity net_etcp_act = {
 	.hdr_line	= "atmptf/s;estres/s;retrans/s;isegerr/s;orsts/s",
 	.name		= "A_NET_ETCP",
 #endif
-	.nr		= &item_nr[14],
+	.nr		= 1,
 	.fsize		= STATS_NET_ETCP_SIZE,
 	.msize		= STATS_NET_ETCP_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
 /* UDP network traffic activity */
 struct activity net_udp_act = {
 	.id		= A_NET_UDP,
-	.options	= AO_CLOSE_MARKUP,
+	.options	= AO_NULL,
 #ifdef SOURCE_SADC
 	.f_count	= NULL,
 	.f_read		= wrap_read_net_udp,
@@ -682,15 +660,252 @@ struct activity net_udp_act = {
 	.hdr_line	= "idgm/s;odgm/s;noport/s;idgmerr/s",
 	.name		= "A_NET_UDP",
 #endif
-	.nr		= &item_nr[14],
+	.nr		= 1,
 	.fsize		= STATS_NET_UDP_SIZE,
 	.msize		= STATS_NET_UDP_SIZE,
 	.opt_flags	= 0,
 	.buf		= {NULL, NULL, NULL},
-	.bitmap		= NULL,
-	.bitmap_size	= 0
+	.bitmap		= NULL
 };
 
+/* IPv6 sockets activity */
+struct activity net_sock6_act = {
+	.id		= A_NET_SOCK6,
+	.options	= AO_NULL,
+#ifdef SOURCE_SADC
+	.f_count	= NULL,
+	.f_read		= wrap_read_net_sock6,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_net_sock6_stats,
+	.f_print_avg	= print_avg_net_sock6_stats,
+#endif
+#ifdef SOURCE_SADF
+	.f_render	= render_net_sock6_stats,
+	.f_xml_print	= xml_print_net_sock6_stats,
+	.hdr_line	= "tcp6sck;udp6sck;raw6sck;ip6-frag",
+	.name		= "A_NET_SOCK6",
+#endif
+	.nr		= 1,
+	.fsize		= STATS_NET_SOCK6_SIZE,
+	.msize		= STATS_NET_SOCK6_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
+
+/* IPv6 network traffic activity */
+struct activity net_ip6_act = {
+	.id		= A_NET_IP6,
+	.options	= AO_NULL,
+#ifdef SOURCE_SADC
+	.f_count	= NULL,
+	.f_read		= wrap_read_net_ip6,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_net_ip6_stats,
+	.f_print_avg	= print_net_ip6_stats,
+#endif
+#ifdef SOURCE_SADF
+	.f_render	= render_net_ip6_stats,
+	.f_xml_print	= xml_print_net_ip6_stats,
+	.hdr_line	= "irec6/s;fwddgm6/s;idel6/s;orq6/s;asmrq6/s;asmok6/s;"
+			  "imcpck6/s;omcpck6/s;fragok6/s;fragcr6/s",
+	.name		= "A_NET_IP6",
+#endif
+	.nr		= 1,
+	.fsize		= STATS_NET_IP6_SIZE,
+	.msize		= STATS_NET_IP6_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
+
+/* IPv6 network traffic (errors) activity */
+struct activity net_eip6_act = {
+	.id		= A_NET_EIP6,
+	.options	= AO_NULL,
+#ifdef SOURCE_SADC
+	.f_count	= NULL,
+	.f_read		= wrap_read_net_eip6,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_net_eip6_stats,
+	.f_print_avg	= print_net_eip6_stats,
+#endif
+#ifdef SOURCE_SADF
+	.f_render	= render_net_eip6_stats,
+	.f_xml_print	= xml_print_net_eip6_stats,
+	.hdr_line	= "ihdrer6/s;iadrer6/s;iukwnp6/s;i2big6/s;idisc6/s;odisc6/s;"
+			  "inort6/s;onort6/s;asmf6/s;fragf6/s;itrpck6/s",
+	.name		= "A_NET_EIP6",
+#endif
+	.nr		= 1,
+	.fsize		= STATS_NET_EIP6_SIZE,
+	.msize		= STATS_NET_EIP6_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
+
+/* ICMPv6 network traffic activity */
+struct activity net_icmp6_act = {
+	.id		= A_NET_ICMP6,
+	.options	= AO_NULL,
+#ifdef SOURCE_SADC
+	.f_count	= NULL,
+	.f_read		= wrap_read_net_icmp6,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_net_icmp6_stats,
+	.f_print_avg	= print_net_icmp6_stats,
+#endif
+#ifdef SOURCE_SADF
+	.f_render	= render_net_icmp6_stats,
+	.f_xml_print	= xml_print_net_icmp6_stats,
+	.hdr_line	= "imsg6/s;omsg6/s;iech6/s;iechr6/s;oechr6/s;igmbq6/s;igmbr6/s;ogmbr6/s;"
+			  "igmbrd6/s;ogmbrd6/s;irtsol6/s;ortsol6/s;irtad6/s;inbsol6/s;onbsol6/s;"
+			  "inbad6/s;onbad6/s",
+	.name		= "A_NET_ICMP6",
+#endif
+	.nr		= 1,
+	.fsize		= STATS_NET_ICMP6_SIZE,
+	.msize		= STATS_NET_ICMP6_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
+
+/* ICMPv6 network traffic (errors) activity */
+struct activity net_eicmp6_act = {
+	.id		= A_NET_EICMP6,
+	.options	= AO_NULL,
+#ifdef SOURCE_SADC
+	.f_count	= NULL,
+	.f_read		= wrap_read_net_eicmp6,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_net_eicmp6_stats,
+	.f_print_avg	= print_net_eicmp6_stats,
+#endif
+#ifdef SOURCE_SADF
+	.f_render	= render_net_eicmp6_stats,
+	.f_xml_print	= xml_print_net_eicmp6_stats,
+	.hdr_line	= "ierr6/s;idtunr6/s;odtunr6/s;itmex6/s;otmex6/s;"
+		          "iprmpb6/s;oprmpb6/s;iredir6/s;oredir6/s;ipck2b6/s;opck2b6/s",
+	.name		= "A_NET_EICMP6",
+#endif
+	.nr		= 1,
+	.fsize		= STATS_NET_EICMP6_SIZE,
+	.msize		= STATS_NET_EICMP6_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
+
+/* UDPv6 network traffic activity */
+struct activity net_udp6_act = {
+	.id		= A_NET_UDP6,
+	.options	= AO_CLOSE_MARKUP,
+#ifdef SOURCE_SADC
+	.f_count	= NULL,
+	.f_read		= wrap_read_net_udp6,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_net_udp6_stats,
+	.f_print_avg	= print_net_udp6_stats,
+#endif
+#ifdef SOURCE_SADF
+	.f_render	= render_net_udp6_stats,
+	.f_xml_print	= xml_print_net_udp6_stats,
+	.hdr_line	= "idgm6/s;odgm6/s;noport6/s;idgmer6/s",
+	.name		= "A_NET_UDP6",
+#endif
+	.nr		= 1,
+	.fsize		= STATS_NET_UDP6_SIZE,
+	.msize		= STATS_NET_UDP6_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
+
+/* CPU frequency */
+struct activity pwr_cpufreq_act = {
+	.id		= A_PWR_CPUFREQ,
+	.options	= AO_NULL,
+#ifdef SOURCE_SADC
+	.f_count	= wrap_get_cpu_nr,
+	.f_read		= wrap_read_cpuinfo,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_pwr_cpufreq_stats,
+	.f_print_avg	= print_avg_pwr_cpufreq_stats,
+#endif
+#ifdef SOURCE_SADF
+	.f_render	= render_pwr_cpufreq_stats,
+	.f_xml_print	= xml_print_pwr_cpufreq_stats,
+	.hdr_line	= "CPU;MHz",
+	.name		= "A_PWR_CPUFREQ",
+#endif
+	.nr		= -1,
+	.fsize		= STATS_PWR_CPUFREQ_SIZE,
+	.msize		= STATS_PWR_CPUFREQ_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= &cpu_bitmap
+};
+
+/* Fan */
+struct activity pwr_fan_act = {
+	.id		= A_PWR_FAN,
+	.options	= AO_NULL,
+#ifdef SOURCE_SADC
+	.f_count	= wrap_get_fan_nr,
+	.f_read		= wrap_read_fan,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_pwr_fan_stats,
+	.f_print_avg	= print_avg_pwr_fan_stats,
+#endif
+#ifdef SOURCE_SADF
+	.f_render	= render_pwr_fan_stats,
+	.f_xml_print	= xml_print_pwr_fan_stats,
+	.hdr_line	= "device;FAN;rpm",
+	.name		= "A_PWR_FAN",
+#endif
+	.nr		= -1,
+	.fsize		= STATS_PWR_FAN_SIZE,
+	.msize		= STATS_PWR_FAN_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
+
+/* Temperature */
+struct activity pwr_temp_act = {
+	.id		= A_PWR_TEMP,
+	.options	= AO_CLOSE_MARKUP,
+#ifdef SOURCE_SADC
+	.f_count	= wrap_get_temp_nr,
+	.f_read		= wrap_read_temp,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_pwr_temp_stats,
+	.f_print_avg	= print_avg_pwr_temp_stats,
+#endif
+#ifdef SOURCE_SADF
+	.f_render	= render_pwr_temp_stats,
+	.f_xml_print	= xml_print_pwr_temp_stats,
+	.hdr_line	= "device;ID;temp_cls",
+	.name	= "A_PWR_TEMP",
+#endif
+	.nr		= -1,
+	.fsize		= STATS_PWR_TEMP_SIZE,
+	.msize		= STATS_PWR_TEMP_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
 
 /*
  * Array of activities.
@@ -718,6 +933,14 @@ struct activity *act[NR_ACT] = {
 	&net_eicmp_act,
 	&net_tcp_act,
 	&net_etcp_act,
-	&net_udp_act
+	&net_udp_act,
+	&net_sock6_act,
+	&net_ip6_act,
+	&net_eip6_act,
+	&net_icmp6_act,
+	&net_eicmp6_act,
+	&net_udp6_act,
+	&pwr_cpufreq_act,
+	&pwr_fan_act,
+	&pwr_temp_act
 };
-
