@@ -1,11 +1,14 @@
 /*
  * sysstat: System performance tools for Linux
- * (C) 1999-2009 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2010 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _COMMON_H
 #define _COMMON_H
 
+/* Maximum length of sensors device name */
+#define MAX_SENSORS_DEV_LEN	20
+
 #include <time.h>
 #include <sched.h>	/* For __CPU_SETSIZE */
 #include "rd_stats.h"
