@@ -16,49 +16,42 @@
 #define I_D_PART_ALL		0x00010
 #define I_D_KILOBYTES		0x00020
 #define I_F_HAS_SYSFS		0x00040
-#define I_F_OLD_KERNEL		0x00080
+#define I_D_DEBUG		0x00080
 #define I_D_UNFILTERED		0x00100
 #define I_D_MEGABYTES		0x00200
 #define I_D_PARTITIONS		0x00400
 #define I_F_HAS_DISKSTATS	0x00800
-#define I_F_HAS_PPARTITIONS	0x01000
-#define I_F_PLAIN_KERNEL24	0x02000
-#define I_D_NFS			0x04000
-#define I_F_HAS_NFS		0x08000
+/* Unused			0x01000 */
+/* Unused			0x02000 */
+/* Unused			0x04000 */
+/* Unused			0x08000 */
 #define I_D_DEVMAP_NAME		0x10000
 #define I_D_ISO			0x20000
-#define I_D_HUMAN_READ		0x40000
+/* Unused			0x40000 */
 #define I_D_ZERO_OMIT		0x80000
 
-#define DISPLAY_CPU(m)		(((m) & I_D_CPU) == I_D_CPU)
-#define DISPLAY_DISK(m)		(((m) & I_D_DISK) == I_D_DISK)
-#define DISPLAY_TIMESTAMP(m)	(((m) & I_D_TIMESTAMP) == I_D_TIMESTAMP)
-#define DISPLAY_EXTENDED(m)	(((m) & I_D_EXTENDED) == I_D_EXTENDED)
-#define DISPLAY_PART_ALL(m)	(((m) & I_D_PART_ALL) == I_D_PART_ALL)
-#define DISPLAY_KILOBYTES(m)	(((m) & I_D_KILOBYTES) == I_D_KILOBYTES)
-#define DISPLAY_MEGABYTES(m)	(((m) & I_D_MEGABYTES) == I_D_MEGABYTES)
-#define HAS_SYSFS(m)		(((m) & I_F_HAS_SYSFS) == I_F_HAS_SYSFS)
-#define HAS_OLD_KERNEL(m)	(((m) & I_F_OLD_KERNEL) == I_F_OLD_KERNEL)
-#define DISPLAY_UNFILTERED(m)	(((m) & I_D_UNFILTERED) == I_D_UNFILTERED)
-#define DISPLAY_PARTITIONS(m)	(((m) & I_D_PARTITIONS) == I_D_PARTITIONS)
+#define DISPLAY_CPU(m)		(((m) & I_D_CPU)           == I_D_CPU)
+#define DISPLAY_DISK(m)		(((m) & I_D_DISK)          == I_D_DISK)
+#define DISPLAY_TIMESTAMP(m)	(((m) & I_D_TIMESTAMP)     == I_D_TIMESTAMP)
+#define DISPLAY_EXTENDED(m)	(((m) & I_D_EXTENDED)      == I_D_EXTENDED)
+#define DISPLAY_PART_ALL(m)	(((m) & I_D_PART_ALL)      == I_D_PART_ALL)
+#define DISPLAY_KILOBYTES(m)	(((m) & I_D_KILOBYTES)     == I_D_KILOBYTES)
+#define DISPLAY_MEGABYTES(m)	(((m) & I_D_MEGABYTES)     == I_D_MEGABYTES)
+#define HAS_SYSFS(m)		(((m) & I_F_HAS_SYSFS)     == I_F_HAS_SYSFS)
+#define DISPLAY_DEBUG(m)	(((m) & I_D_DEBUG)         == I_D_DEBUG)
+#define DISPLAY_UNFILTERED(m)	(((m) & I_D_UNFILTERED)    == I_D_UNFILTERED)
+#define DISPLAY_PARTITIONS(m)	(((m) & I_D_PARTITIONS)    == I_D_PARTITIONS)
 #define HAS_DISKSTATS(m)	(((m) & I_F_HAS_DISKSTATS) == I_F_HAS_DISKSTATS)
-#define HAS_PPARTITIONS(m)	(((m) & I_F_HAS_PPARTITIONS) == I_F_HAS_PPARTITIONS)
-#define HAS_PLAIN_KERNEL24(m)	(((m) & I_F_PLAIN_KERNEL24) == I_F_PLAIN_KERNEL24)
-#define DISPLAY_NFS(m)		(((m) & I_D_NFS) == I_D_NFS)
-#define HAS_NFS(m)		(((m) & I_F_HAS_NFS) == I_F_HAS_NFS)
-#define DISPLAY_DEVMAP_NAME(m)	(((m) & I_D_DEVMAP_NAME) == I_D_DEVMAP_NAME)
-#define DISPLAY_ISO(m)		(((m) & I_D_ISO) == I_D_ISO)
-#define DISPLAY_HUMAN_READ(m)	(((m) & I_D_HUMAN_READ) == I_D_HUMAN_READ)
-#define DISPLAY_ZERO_OMIT(m)	(((m) & I_D_ZERO_OMIT) == I_D_ZERO_OMIT)
+#define DISPLAY_DEVMAP_NAME(m)	(((m) & I_D_DEVMAP_NAME)   == I_D_DEVMAP_NAME)
+#define DISPLAY_ISO(m)		(((m) & I_D_ISO)           == I_D_ISO)
+#define DISPLAY_ZERO_OMIT(m)	(((m) & I_D_ZERO_OMIT)     == I_D_ZERO_OMIT)
 
 /* Preallocation constats */
 #define NR_DEV_PREALLOC		4
 #define NR_DISK_PREALLOC	3
-#define NR_NFS_PREALLOC		2
-
-/* Device name for old kernels */
-#define K_HDISK	"hdisk"
 
+/* Environment variable */
+#define ENV_POSIXLY_CORRECT	"POSIXLY_CORRECT"
 
 /*
  * Structures for I/O stats.
@@ -107,20 +100,6 @@ struct io_stats {
 
 #define IO_STATS_SIZE	(sizeof(struct io_stats))
 
-struct io_nfs_stats {
-	unsigned long long rd_normal_bytes	__attribute__ ((aligned (8)));
-	unsigned long long wr_normal_bytes	__attribute__ ((packed));
-	unsigned long long rd_direct_bytes	__attribute__ ((packed));
-	unsigned long long wr_direct_bytes	__attribute__ ((packed));
-	unsigned long long rd_server_bytes	__attribute__ ((packed));
-	unsigned long long wr_server_bytes	__attribute__ ((packed));
-	unsigned long rpc_sends			__attribute__ ((packed));
-	unsigned long nfs_rops			__attribute__ ((packed));
-	unsigned long nfs_wops			__attribute__ ((packed));
-};
-
-#define IO_NFS_STATS_SIZE	(sizeof(struct io_nfs_stats))
-
 struct io_hdr_stats {
 	unsigned int active		__attribute__ ((aligned (4)));
 	unsigned int used		__attribute__ ((packed));
