@@ -1,6 +1,6 @@
 /*
  * pr_stats.c: Functions used by sar to display statistics
- * (C) 1999-2009 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2010 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -1764,3 +1764,187 @@ __print_funct_t print_avg_pwr_cpufreq_stats(struct activity *a, int prev, int cu
 {
 	stub_print_pwr_cpufreq_stats(a, prev, curr, TRUE);
 }
+
+/*
+ ***************************************************************************
+ * Display fan statistics. This function is used to display
+ * instantaneous and average statistics.
+ *
+ * IN:
+ * @a          Activity structure with statistics.
+ * @prev       Index in array where stats used as reference are.
+ * @curr       Index in array for current sample statistics.
+ * @dispavg    True if displaying average statistics.
+ ***************************************************************************
+ */
+void stub_print_pwr_fan_stats(struct activity *a, int prev, int curr, int dispavg)
+{
+	int i;
+	struct stats_pwr_fan *spc;
+	static double *avg_fan = NULL;
+
+	if (!avg_fan) {
+		/* Allocate array of fan RPMs */
+		if ((avg_fan = (double *) malloc(sizeof(double) * a->nr)) == NULL) {
+			perror("malloc");
+			exit(4);
+		}
+		memset(avg_fan, 0, sizeof(double) * a->nr);
+	}
+
+	if (dis) {
+		printf("\n%-11s     FAN       rpm     %*s\n",
+		       timestamp[!curr], MAX_SENSORS_DEV_LEN, "DEVICE");
+	}
+
+	for (i = 0; (i < a->nr); i++) {
+		spc = (struct stats_pwr_fan *) ((char *) a->buf[curr] + i * a->msize);
+
+		printf("%-11s     %3d", timestamp[curr], i + 1);
+
+		if (dispavg) {
+			/* Display average values */
+			printf(" %9.2f", (double) avg_fan[i] / avg_count);
+		}
+		else {
+			/* Display instantaneous values */
+			printf(" %9.2f", spc->rpm);
+			avg_fan[i] += spc->rpm;
+		}
+
+		printf("     %*s\n", MAX_SENSORS_DEV_LEN, spc->device);
+	}
+
+	if (dispavg) {
+		if (avg_fan) {
+			free(avg_fan);
+			avg_fan = NULL;
+		}
+	}
+}
+
+/*
+ ***************************************************************************
+ * Display fan statistics.
+ *
+ * IN:
+ * @a          Activity structure with statistics.
+ * @prev       Index in array where stats used as reference are.
+ * @curr       Index in array for current sample statistics.
+ * @itv                Interval of time in jiffies.
+ ***************************************************************************
+ */
+__print_funct_t print_pwr_fan_stats(struct activity *a, int prev, int curr,
+				    unsigned long long itv)
+{
+	stub_print_pwr_fan_stats(a, prev, curr, FALSE);
+}
+
+/*
+ ***************************************************************************
+ * Display average fan statistics.
+ *
+ * IN:
+ * @a          Activity structure with statistics.
+ * @prev       Index in array where stats used as reference are.
+ * @curr       Index in array for current sample statistics.
+ * @itv                Interval of time in jiffies.
+ ***************************************************************************
+ */
+__print_funct_t print_avg_pwr_fan_stats(struct activity *a, int prev, int curr,
+                                       unsigned long long itv)
+{
+	stub_print_pwr_fan_stats(a, prev, curr, TRUE);
+}
+
+/*
+ ***************************************************************************
+ * Display device temperature statistics. This function is used to display
+ * instantaneous and average statistics.
+ *
+ * IN:
+ * @a          Activity structure with statistics.
+ * @prev       Index in array where stats used as reference are.
+ * @curr       Index in array for current sample statistics.
+ * @dispavg    True if displaying average statistics.
+ ***************************************************************************
+ */
+void stub_print_pwr_temp_stats(struct activity *a, int prev, int curr, int dispavg)
+{
+	int i;
+	struct stats_pwr_temp *spc;
+	static double *avg_temp = NULL;
+
+	if (!avg_temp) {
+		/* Allocate array of temperatures */
+		if ((avg_temp = (double *) malloc(sizeof(double) * a->nr)) == NULL) {
+			perror("malloc");
+			exit(4);
+		}
+		memset(avg_temp, 0, sizeof(double) * a->nr);
+	}
+
+	if (dis) {
+		printf("\n%-11s      ID  temp_cls     %*s\n",
+		       timestamp[!curr], MAX_SENSORS_DEV_LEN, "DEVICE");
+	}
+
+	for (i = 0; (i < a->nr); i++) {
+		spc = (struct stats_pwr_temp *) ((char *) a->buf[curr] + i * a->msize);
+
+		printf("%-11s     %3d", timestamp[curr], i + 1);
+
+		if (dispavg) {
+			/* Display average values */
+			printf(" %9.2f", (double) avg_temp[i] / avg_count);
+		}
+		else {
+			/* Display instantaneous values */
+			printf(" %9.2f", spc->temp);
+			avg_temp[i] += spc->temp;
+		}
+		
+		printf("     %*s\n", MAX_SENSORS_DEV_LEN, spc->device);
+	}
+
+	if (dispavg) {
+		if (avg_temp) {
+			free(avg_temp);
+			avg_temp = NULL;
+		}
+	}
+}
+
+/*
+ ***************************************************************************
+ * Display temperature statistics.
+ *
+ * IN:
+ * @a          Activity structure with statistics.
+ * @prev       Index in array where stats used as reference are.
+ * @curr       Index in array for current sample statistics.
+ * @itv                Interval of time in jiffies.
+ ***************************************************************************
+ */
+__print_funct_t print_pwr_temp_stats(struct activity *a, int prev, int curr,
+                                       unsigned long long itv)
+{
+	stub_print_pwr_temp_stats(a, prev, curr, FALSE);
+}
+
+/*
+ ***************************************************************************
+ * Display average temperature statistics.
+ *
+ * IN:
+ * @a          Activity structure with statistics.
+ * @prev       Index in array where stats used as reference are.
+ * @curr       Index in array for current sample statistics.
+ * @itv                Interval of time in jiffies.
+ ***************************************************************************
+ */
+__print_funct_t print_avg_pwr_temp_stats(struct activity *a, int prev, int curr,
+                                       unsigned long long itv)
+{
+	stub_print_pwr_temp_stats(a, prev, curr, TRUE);
+}
