@@ -1,6 +1,6 @@
 /*
  * sar/sadc: Report system activity
- * (C) 1999-2009 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2010 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _SA_H
@@ -17,7 +17,7 @@
  */
 
 /* Number of activities */
-#define NR_ACT	30
+#define NR_ACT	32
 
 /* Activities */
 #define A_CPU		1
@@ -50,6 +50,8 @@
 #define A_NET_EICMP6	28
 #define A_NET_UDP6	29
 #define A_PWR_CPUFREQ	30
+#define A_PWR_FAN	31
+#define A_PWR_TEMP	32
 
 
 /* Macro used to flag an activity that should be collected */
@@ -145,6 +147,9 @@
 #define K_IPV6		"IPV6"
 #define K_POWER		"POWER"
 #define K_XDISK		"XDISK"
+#define K_CPU		"CPU"
+#define K_FAN		"FAN"
+#define K_TEMP		"TEMP"
 
 /* sadc program */
 #define SADC		"sadc"
@@ -442,7 +447,7 @@ struct file_header {
 	unsigned int sa_nr_act		__attribute__ ((aligned (8)));
 	/*
 	 * Current day, month and year.
-	 * No need to save DST (daylight saving time) flag, since it is not taken
+	 * No need to save DST (Daylight Saving Time) flag, since it is not taken
 	 * into account by the strftime() function used to print the timestamp.
 	 */
 	unsigned char sa_day;
@@ -600,6 +605,10 @@ extern __nr_t
 	wrap_get_disk_nr(struct activity *);
 extern __nr_t
 	wrap_get_iface_nr(struct activity *);
+extern __nr_t
+	wrap_get_fan_nr(struct activity *);
+extern __nr_t
+	wrap_get_temp_nr(struct activity *);
 
 /* Functions used to read activities statistics */
 extern __read_funct_t
@@ -662,6 +671,10 @@ extern __read_funct_t
 	wrap_read_net_udp6(struct activity *);
 extern __read_funct_t
 	wrap_read_cpuinfo(struct activity *);
+extern __read_funct_t
+	wrap_read_fan(struct activity *);
+extern __read_funct_t
+	wrap_read_temp(struct activity *);
 
 /* Other functions */
 extern void
@@ -710,6 +723,8 @@ extern int
 	parse_sar_I_opt(char * [], int *, struct activity * []);
 extern int
 	parse_sa_P_opt(char * [], int *, unsigned int *, struct activity * []);
+extern int
+	parse_sar_m_opt(char * [], int *, struct activity * []);
 extern int
 	parse_sar_n_opt(char * [], int *, struct activity * []);
 extern int
